###################################################################
#  GTChat 0.95 Alpha Plugin                                       #
#  Written for release 20040112                                   #
#  Author: Norman Rath                                            #
#  E-Mail: Norman.Rath@o2online.de                                #
#                                                                 #
#  This plugin send a newsletter to the chatuser                  #
###################################################################

package GTChat::Plugins::Administration::Newsletter::095_01;
use strict;

return bless({
	action_handlers => {
		'admin_newsletter' => \&handler,
		'admin_newsletterprocessmembers' => \&process_handler,
	},
});

sub handler
{
	my($self,$main) = @_;
	
	$main->{template_vars}{backaddr} = 'javascript:history.back()';

	$main->fatal_error('nopermission') unless $main->hasPermission('admin_newsletter');

	if(!$main->{input}{subject} || !$main->{input}{message})
	{
		$main->{template_vars}{error_subject} = 1 if(!$main->{input}{subject});
		$main->{template_vars}{error_message} = 1 if(!$main->{input}{message});

		$main->printTemplate('admin/newsletter');
	}
	else
	{
		my $users = $main->getAllUsers();

		my(@userlist,@grouplist) = ();
		my $count_total = 0;

		for (my $i = 0; $i <= $#$users; $i++)
		{
			my($nickname,$username) = split(/\|/,$users->[$i]);
			my $user = $main->loadUser($username);

			next if($user->{newsletter} ne '1' && $main->{input}{ignoresettings} eq '1');

			if($main->{input}{sendto} eq '0')
			{
				next if($user->{group} < $main->{input}{group});
			}
			elsif($main->{input}{sendto} eq '1')
			{
				next if($user->{group} > $main->{input}{group});
			}
			elsif($main->{input}{sendto} eq '2')
			{
				next if($user->{group} ne $main->{input}{group});
			}
			elsif($main->{input}{sendto} eq '3')
			{
				next if($user->{group} eq $main->{input}{group});
			}

			$count_total++;

			push(@userlist,$user->{name}."\n");
			push(@grouplist,$user->{group}."\n");
		}

		if ($#userlist >= 0 && $#grouplist >= 0)
		{
			$main->open(local* FILE, '>'.$main->translateName('tmpdir::newsletter_grouplist_queue')) || $main->fatal_error('couldnotcreate', {file => $main->translateName('tmpdir::newsletter_grouplist_queue')});
			print FILE @grouplist;
			$main->close(*FILE);

			$main->open(local* FILE, '>'.$main->translateName('tmpdir::newsletter_userlist_queue')) || $main->fatal_error('couldnotcreate', {file => $main->translateName('tmpdir::newsletter_userlist_queue')});
			print FILE @userlist;
			$main->close(*FILE);

			$main->{template_vars}{processed} = 0;
			$main->{template_vars}{count_total} = $count_total;

			$main->{input}{message} =~ s/\r//g;
			$main->{input}{message} =~ s/\n/\[br\]/g;

			$main->open(local* FILE, '>'.$main->translateName('tmpdir::newsletter_temp')) || $main->fatal_error('couldnotcreate', {file => $main->translateName('tmpdir::newsletter_temp')});
			print FILE $main->{input}{subject}."\n".$main->{input}{message};
			$main->close(*FILE);

			$main->printTemplate('admin/newsletter_processing');
		}
		else
		{
			$main->printTemplate('admin/newsletter_evaluation');
		}
	}
}

sub process_handler
{
	my($self,$main) = @_;

	$main->{template_vars}{backaddr} = 'javascript:history.back()';

	$main->fatal_error('nopermission') unless $main->hasPermission('admin_newsletter');

	$main->open(local* FILE, $main->translateName('tmpdir::newsletter_userlist_queue')) || $main->fatal_error('couldnotopen', {file => $main->translateName('tmpdir::newsletter_userlist_queue')});
	my @userlist = <FILE>;
	$main->close(*FILE);

	my $count_now = 0;
	my $maxcount = ($main->{input}{count_total} ? 10 : 30);
	my @count_group = ();

	$main->open(local* FILE, $main->translateName('tmpdir::newsletter_temp')) || $main->fatal_error('couldnotopen', {file => $main->translateName('tmpdir::newsletter_temp')});
	my @newsletter = <FILE>;
	$main->close(*FILE);

	my $newsletter;

	$newsletter->{subject} = @newsletter[0];
	$newsletter->{subject} =~ s/\n//g;
	$newsletter->{message} = @newsletter[1];
	$newsletter->{message} =~ s/\[br\]/\n/g;

	foreach my $name (@userlist)
	{
		$name =~ s/[\r\n]//g;

		my $user = $main->loadUser($name);

		$newsletter->{toemail} = $user->{email};
		$newsletter->{tonick} = $user->{nick};

		$main->{template_vars}{newsletter} = $newsletter;
		$main->sendMailTemplate('mails/newsletter');

		$count_now++;

		last unless $count_now < $maxcount;
	}

	if ($#userlist < $maxcount)
	{
		unlink($main->translateName('tmpdir::newsletter_userlist_queue'));

		$main->open(local* FILE, $main->translateName('tmpdir::newsletter_grouplist_queue')) || $main->fatal_error('couldnotopen', {file => $main->translateName('tmpdir::newsletter_grouplist_queue')});
		my @grouplist = <FILE>;
		$main->close(*FILE);

		foreach my $group (@grouplist)
		{
			$group =~ s/[\r\n]//g;

			$count_group[$group]++;
		}

		unlink($main->translateName('tmpdir::newsletter_grouplist_queue'));
		unlink($main->translateName('tmpdir::newsletter_temp'));

		$main->{template_vars}{count_total} = $main->{input}{count_total};
		$main->{template_vars}{count_group} = \@count_group;
		$main->printTemplate('admin/newsletter_evaluation');
	}
	else
	{
		sleep(5) if($main->{input}{count_total});

		splice(@userlist, 0, $maxcount);
		$main->open(local* FILE, '>'.$main->translateName('tmpdir::newsletter_userlist_queue')) || $main->fatal_error('couldnotcreate', {file => $main->translateName('tmpdir::newsletter_userlist_queue')});
		print FILE @userlist;
		$main->close(*FILE);

		$main->{template_vars}{processed} = $main->{input}{count_total} - $#userlist - 1;
		$main->{template_vars}{count_total} = $main->{input}{count_total};
		$main->printTemplate('admin/newsletter_processing');
	}
}
