###################################################################
#  GTChat 0.95 Alpha Plugin                                       #
#  Written for release 20021101                                   #
#  Author: Norman Rath                                            #
#                                                                 #
#  This plugin provides the call user command /calluser           #
###################################################################

package GTChat::Plugins::CallUserCommand;
use strict;

return bless({
	command_handlers => {
		'calluser' => \&commandHandler,
	},
});

sub check_gagged
{
	my $main = shift;

	if (exists($main->{current_user}{gagtime}) && $main->{runtime}{now} < $main->{current_user}{gagtime})
	{
		return [$main->createErrorOutput('gagged',{seconds => $main->{current_user}{gagtime}-$main->{runtime}{now}})];
	}
	return undef;
}

sub commandHandler
{
	my($self,$main,$command,$text) = @_;

# Send Last_Call berprfung

	my $user = $main->loadUser($main->{input}{username});

	if(($main->{runtime}{now} - $main->{current_user}{last_call}) < 30)
	{
	return [$main->createErrorOutput('too_many_calls')];
	}

	$main->{current_user}{last_call} = $main->{runtime}{now};
	$main->saveUser($main->{current_user});

# Ende Last_Call berprfung

	if (my $error=check_gagged($main))
	{
		return $error;
	}
	
	my @parts = split(/\s+/,$text);
	my $tonick = shift @parts;

	return [$main->createErrorOutput('msg_namenotgiven')] if (!defined($tonick));

	my $candidates = $main->getPossibleOnlineUsers($tonick);

	if ($#$candidates>0)    # Too many users online with nick starting like this
	{
		return [$main->createErrorOutput('ambiguousname',{nick => $tonick})];
	}
	elsif ($#$candidates==0)   # User is online
	{
		my $user = $candidates->[0];
		
		return [$main->createErrorOutput('msgtooneself')] if ($user->{name} eq $main->{current_user}{name});

		my $output1 = $main->createOutput(
					{
						template => 'calluser',
						toname => $user->{name},
						tonick => $user->{nick},
						color => $main->{current_user}{color},
,						calluser => '1',
					});
		my $output2 = $main->createOutput(
					{
						template => 'calluser',
						name => $main->{current_user}{name},
						nick => $main->{current_user}{nick},
						color => $main->{current_user}{color},
						calluser => '',
					});
		
		return [$output1->restrictToCurrentUser, $output2->restrictToUser($user->{name})];
	}
	else                      # No user with this nickname online
	{
		return [$main->createErrorOutput('notonline',{nick => $tonick})];
	}
}

