###################################################################
#  GTChat 0.95 Alpha Plugin                                       #
#  Written for release 20030110                                   #
#  Author: Norman Rath                                            #
#                                                                 #
#  This plugin provides the follow user command /follow           #
###################################################################

package GTChat::Plugins::FollowCommand;
use strict;

return bless({
	command_handlers => {
		'follow' => \&commandHandler,
	},
});

sub commandHandler
{
	my($self,$main,$command,$text) = @_;

	my @parts = split(/\s+/,$text);
	my $tonick = shift @parts;

	return [$main->createErrorOutput('follow_namenotgiven')] if (!defined($tonick));

	my $candidates = $main->getPossibleOnlineUsers($tonick);

	if ($#$candidates>0)
	{
		return [$main->createErrorOutput('ambiguousname',{nick => $tonick})];
	}
	elsif ($#$candidates==0)
	{
		my $user = $candidates->[0];
		
		return [$main->createErrorOutput('follow_tooneself')] if ($user->{name} eq $main->{current_user}{name});

		my $room = $main->loadRoom($user->{room});

		if (!defined($room) || !$main->isRoomPermitted($room))
		{
			return [$main->createErrorOutput('unknownroom',{roomname => $user->{room}})];
		}

		if ($room->{name_lc} ne $main->{current_user}{room})
		{
			if ($room->{max_users})
			{
				my $count = 0;
				my $users = $main->getOnlineUsers();
				my $user = $main->{current_user};
				foreach (@$users)
				{
					$count++ if ($room->{name_lc} eq (split(/\|/))[3]);
				}
				if ($count >= $room->{max_users})
				{
					$main->fatal_error('max_room_users',{maxcount => $room->{max_users}}) if($main->{settings}{permissions}{ignore_max_users} > $user->{tempgroup} && !$main->hasPermission('ignore_max_users',$user));
				}
			}

			my $oldroom = $main->{current_user}{room};
			$main->{current_user}{room} = $user->{room};

			my $output1 = $main->createOutput(
			{
				template => 'room_leaved',
				name => $main->{current_user}{name},
				nick => $main->{current_user}{nick},
				fromroom => $oldroom,
				toroom => $main->{current_user}{room},
			});
			$output1->restrictToRoom($oldroom);

			my $output2 = $main->createOutput(
			{
				template => 'room_entered',
				name => $main->{current_user}{name},
				nick => $main->{current_user}{nick},
				fromroom => $oldroom,
				toroom => $main->{current_user}{room},
			});
			$output2->restrictToRoom($main->{current_user}{room});

			my $output3 = $main->createOutput(
			{
				template => 'room_greeting',
				name => $main->{current_user}{name},
				nick => $main->{current_user}{nick},
				fromroom => $oldroom,
				toroom => $main->{current_user}{room},
			});
			$output3->restrictToUser($main->{current_user}{name});
			$output3->setChangedAttributes('room');
		
			return [$output1, $output2, $output3];
		}

		return undef;
	}
	else
	{
		return [$main->createErrorOutput('notonline',{nick => $tonick})];
	}
}

