###################################################################
#  GTChat 0.95 Alpha Plugin                                       #
#  Written for release 20021101                                   #
#  Author: Norman Rath                                            #
#                                                                 #
#  This plugin provides the font command /font                    #
###################################################################

package GTChat::Plugins::FontCommand;
use strict;

return bless({
	command_handlers => {
		'font' => \&commandHandler,
	},
});

sub check_gagged
{
	my $main = shift;

	if (exists($main->{current_user}{gagtime}) && $main->{runtime}{now} < $main->{current_user}{gagtime})
	{
		return [$main->createErrorOutput('gagged',{seconds => $main->{current_user}{gagtime}-$main->{runtime}{now}})];
	}
	return undef;
}

sub commandHandler
{
	my($self,$main,$command,$text) = @_;

	my @ret=();

	if (my $error=check_gagged($main))
	{
		return $error;
	}
	
	if ($main->{current_user}{away})
	{
		$main->{current_user}{away} = 0;
		
		my $output = $main->createInfoOutput('awayoff',{nick => $main->{current_user}{nick}});
		push @ret, $output->restrictToUser->restrictToCurrentRoom->setChangedAttributes('away');
	}

	my(@parts) = split(/\s/,$text);
	my $fontsize = shift @parts;
	my $tmptext = $fontsize;
	$text = join(' ',@parts);
	$fontsize = 0 unless defined($fontsize) && $fontsize>0 && $fontsize<8;
	if($fontsize == 0)
	{
		$text = $tmptext.' '.$text;
	}

	return \@ret if ($text eq '' || $text eq ' ');

	my $output = $main->createOutput(
		{
			template => 'font',
			name => $main->{current_user}{name},
			nick => $main->{current_user}{nick},
			color => $main->{current_user}{color},
			fontsize => $fontsize,
			text => $main->toHTML($text),
		});
	push @ret, $output->restrictToCurrentRoom;

	return \@ret;
}

