###################################################################
#  GTChat 0.95 Alpha Plugin                                       #
#  Written for release 20040112                                   #
#  Author: Norman Rath                                            #
#  E-Mail: Norman.Rath@o2online.de                                #
#                                                                 #
#  This plugin provides the command /lock, /unlock and /viewlock  #
###################################################################

package GTChat::Plugins::LockCommand;
use strict;

return bless({
        command_handlers => {
                'lock' => \&commandHandler,
                'unlock' => \&commandHandler,
		'viewlock' => \&commandHandlerViewlock,
        },
});

sub commandHandler
{
        my($self,$main,$command,$text) = @_;

	my(@parts) = split(/\s/,$text);
	my $nick = shift @parts;
	my $reason = join(' ',@parts);

	return [$main->createErrorOutput($command.'_namenotgiven')] unless defined($nick);

	my $candidates = $main->getPossibleUsernames($nick);
		
	$main->fatal_error('ambiguousname',{nick => $nick}) if ($#$candidates>1);
	$main->fatal_error('unknownname',{nick => $nick}) if ($#$candidates<0);

	my $user = $main->loadUser($candidates->[0]);

	if($command eq 'lock')
	{
		my $candidates = $main->getPossibleOnlineUsers($nick);

		if($#$candidates == 0)
		{
			return [$main->createErrorOutput($command.'_nopermission',{nick => $user->{nick}})] if ($main->{current_user}{tempgroup} <= $user->{tempgroup});
			return [$main->createErrorOutput($command.'_admin',{nick => $user->{nick}})] if ($user->{group} >= $main->{current_user}{tempgroup});
		}
		else
		{
			return [$main->createErrorOutput($command.'_nopermission',{nick => $user->{nick}})] if ($main->{current_user}{tempgroup} <= $user->{group});
			return [$main->createErrorOutput($command.'_admin',{nick => $user->{nick}})] if ($user->{group} >= $main->{current_user}{tempgroup});
		}

	        $main->open(local *FILE,'>>'.$main->translateName('vardir::banlist.txt'));
	        print FILE "$user->{host}|$user->{ip}|$user->{forwardedfor}|$main->{runtime}{now}|$reason|$user->{nick}|$user->{name}\n";
	        $main->close(*FILE);

		if($#$candidates == 0)
		{
			my $user = $candidates->[0];

			$main->kickUser($candidates->[0],{error_name => 'error_locked',reason => $reason,nologoutmsg => 1});

			my $output = $main->createInfoOutput('locked_online',{admin => $main->{current_user}{nick},nick => $user->{nick},reason => $reason});
			return [$output->restrictToRoom($user->{room})];
		}
		else
		{
			my $output = $main->createInfoOutput('locked',{nick => $user->{nick},});
			return [$output->restrictToCurrentUser];
		}
	}
	else
	{
		$main->open(local *FILE,'+>>'.$main->translateName('vardir::banlist.txt')) || $main->fatal_error('couldnotopen',{file => $main->translateName('vardir::banlist.txt')});
		seek(FILE,0,0);

		my @users=<FILE>;
	
		my $min = 0;
		my $max = $#users;
	
		while ($min <= $max)
		{
			my $curindex = int(($min+$max)/2);
			my ($curhost,$curip,$curforwardedfor,$curtime,$curreason,$curnick,$curname) = split(/\|/,$users[$curindex]);

			$curname =~ s/[\n]//g;

			my $result = $curname cmp $user->{name};

			if ($result == 0)
			{
				splice(@users,$curindex,1);
				last;
			}
			elsif ($result < 0)
			{
				$min = $curindex + 1;
			}
			else
			{
				$max = $curindex - 1;
			}
		}

		if ($min > $max)
		{
			$main->close(*FILE);

			my $output = $main->createErrorOutput('unlock_notfound',{nick => $user->{nick},});
			return [$output->restrictToCurrentUser];
		}
	
		seek(FILE,0,0);
		truncate(FILE,0);
		print FILE foreach @users;

		$main->close(*FILE);

		my $output = $main->createInfoOutput('unlocked',{nick => $user->{nick},});
		return [$output->restrictToCurrentUser];
	}
}

sub commandHandlerViewlock
{
        my($self,$main) = @_;

	$main->open(local* FILE, $main->translateName('vardir::banlist.txt')) || $main->fatal_error('couldnotopen',{file => $main->translateName('vardir::banlist.txt')});
	my @list=<FILE>;
	$main->close(*FILE);

	my @users=();

	foreach (@list)
	{
		my ($host,$ip,$forwardedfor,$time,$reason,$name) = split(/\|/,$_);

		$name =~ s/[\n]//g;

		my $user = $main->loadUser($name);
		push @users, $user->{nick} if $user;
	}

	if ($#users < 0)
	{
		return [$main->createInfoOutput('lock_none')];
	}
	else
	{
		return [$main->createInfoOutput('lock_users',{list => join(', ',@users)})];
	}
}
