###################################################################
#  GTChat 0.95 Alpha Plugin                                       #
#  Written for release 20021101                                   #
#  Author: Marcus Kristen                                         #
###################################################################

package GTChat::Plugins::PointCommand;
use strict;

return bless({
        command_handlers => {
                'point' => \&point_handler,
                'minus' => \&minus_handler,
                'null' => \&null_handler,
        },
});

sub point_handler
{
        my($self,$main,$command,$text) = @_;

        my ($nick,$points) = split(/\s+/,$text);

        return [$main->createErrorOutput('kick_namenotgiven')] unless defined($nick);

        my $candidates = $main->getPossibleOnlineUsers($nick);

        return [$main->createErrorOutput('notonline',{nick => $nick})] if ($#$candidates < 0);
        return [$main->createErrorOutput('ambiguousname',{nick => $nick})] if ($#$candidates > 0);

        my $user = $candidates->[0];

        $points = 10 if $points<=0;

        $main->loadUser($user->{name},$user);

        $user->{points} = $user->{points} + $points;

        $main->saveUser($user);


        my $info = $main->createInfoOutput('point',
                                        {
                                                admin => $main->{current_user}{nick},
                                                nick => $user->{nick},
                                                points => $points,
                                        });
        $info->restrictToRoom($user->{room});

        return [$info];
}

sub minus_handler
{
        my($self,$main,$command,$text) = @_;

        my ($nick,$points) = split(/\s+/,$text);

        return [$main->createErrorOutput('kick_namenotgiven')] unless defined($nick);

        my $candidates = $main->getPossibleOnlineUsers($nick);

        return [$main->createErrorOutput('notonline',{nick => $nick})] if ($#$candidates < 0);
        return [$main->createErrorOutput('ambiguousname',{nick => $nick})] if ($#$candidates > 0);

        my $user = $candidates->[0];

        $points = 10 if $points<=0;

        $main->loadUser($user->{name},$user);

        $user->{points} = $user->{points} - $points;

        $main->saveUser($user);


        my $info2 = $main->createInfoOutput('minus',
                                        {
                                                admin => $main->{current_user}{nick},
                                                nick => $user->{nick},
                                                points => $points,
                                        });
        $info2->restrictToRoom($user->{room});

        return [$info2];
}
sub null_handler
{
        my($self,$main,$command,$text) = @_;

        my ($nick,$points) = split(/\s+/,$text);

        return [$main->createErrorOutput('kick_namenotgiven')] unless defined($nick);

        my $candidates = $main->getPossibleOnlineUsers($nick);

        return [$main->createErrorOutput('notonline',{nick => $nick})] if ($#$candidates < 0);
        return [$main->createErrorOutput('ambiguousname',{nick => $nick})] if ($#$candidates > 0);

        my $user = $candidates->[0];

        $points = 10 if $points<=0;

        $main->loadUser($user->{name},$user);

        $user->{points} = 0;

        $main->saveUser($user);


        my $info3 = $main->createInfoOutput('null',
                                        {
                                                admin => $main->{current_user}{nick},
                                                nick => $user->{nick},
                                                points => $points,
                                        });
        $info3->restrictToRoom($user->{room});

        return [$info3];
}