###################################################################
#  GTChat 0.95 Alpha Plugin                                       #
#  Written for release 20030110                                   #
#  Author: Norman Rath                                            #
#                                                                 #
#  This plugin provides the topic command /topic                  #
###################################################################

package GTChat::Plugins::TopicCommand;
use strict;

return bless({
	command_handlers => {
		'topic' => \&commandHandler,
	},
});

sub check_gagged
{
	my $main = shift;

	if(exists($main->{current_user}{gagtime}) && $main->{runtime}{now} < $main->{current_user}{gagtime})
	{
		return [$main->createErrorOutput('gagged',{seconds => $main->{current_user}{gagtime}-$main->{runtime}{now}})];
	}
	return undef;
}

sub commandHandler
{
	my($self,$main,$command,$text) = @_;

	if(my $error = check_gagged($main))
	{
		return $error;
	}
	
	my $room = $main->loadRoom($main->{current_user}{room});

	return [$main->createInfoOutput('topic',{text => $room->{topic},})] if($text eq '');

	my $user = $main->{current_user};

	if($main->{current_user}{name} ne $room->{owner} && $room->{owner} ne '')
	{
		return [$main->createErrorOutput('roomlist_nopermission')] if($main->{settings}{permissions}{rooms_modify} > $user->{tempgroup} && !$main->hasPermission('rooms_modify',$user));
	}

	return [$main->createErrorOutput('max_topic_len')] if(length($main->toHTML($text)) > $main->{settings}{check_room_fields_length}{topic}[0]);

	$room->{topic} = $main->toHTML($text).' ('.$main->{current_user}{nick}.')';
	$main->saveRoom($room);

	my $output = $main->createInfoOutput('topic_changed',{nick => $main->{current_user}{nick},text => $main->toHTML($text),});
	return [$output->restrictToCurrentRoom];
}

