###################################################################
#  GTChat 0.95 Alpha Plugin                                       #
#  Written for release 20030110                                   #
#  Author: Sven Schmidt                                        #
###################################################################

package GTChat::Plugins::WhoisCommand;
use strict;

return bless({
        command_handlers => {
                'whois' => \&whoisHandler,
        },
});

sub whoisHandler
{
        my($self,$main,$command,$text) = @_;

        my @parts = split(/\s+/,$text);
        my $tonick = shift @parts;
        $text = $main->toHTML(join(' ',@parts));

        return [$main->createErrorOutput('msg_namenotgiven')] if (!defined($tonick));

        my $candidates = $main->getPossibleOnlineUsers($tonick);

        if ($#$candidates>0)    # Too many users online with nick starting like this
        {
                return [$main->createErrorOutput('ambiguousname',{nick => $tonick})];
        }
        elsif ($#$candidates==0)   # User is online
        {
                my $user = $candidates->[0];

                $main->loadUser($user->{name},$user);

                my $outputon = $main->createOutput(
                                        {
                                                template => 'whois',
                                                nick => $user->{nick},
                                                raumton => 'ONLINE',
                                                raum => $user->{room},
                                                name => $user->{name},
                                                away => $user->{away},
                                                group => $user->{tempgroup},
                                                lastlogin => $user->{lastlogin},
                                                host => $user->{host},
                                                browser => $user->{browser},
                                                ip => $user->{ip},
                                                pmail => $user->{privateemail},
                                                email => $user->{email},
                                        });
                return [$outputon->restrictToCurrentUser];
        }
        else                      # No user with this nickname online
        {

                $candidates = $main->getPossibleUsernames($tonick);

                if ($#$candidates>1)        # There are too many users with nick starting like this
                {
                        return [$main->createErrorOutput('ambiguousname',{nick => $tonick})];
                }
                elsif ($#$candidates<0)     # No user with this nick found
                {
                        return [$main->createErrorOutput('unknownname',{nick => $tonick})];
                }
                else                        # User was found but isn't online
                {
                        my $toname = $candidates->[0];
                        $tonick = $candidates->[1];

                my $user = $main->loadUser($toname);

                my $outputoff = $main->createOutput(
                                        {
                                                template => 'whois',
                                                nick => $user->{nick},
                                                raumtoff => 'OFFLINE',
                                                name => $user->{name},
                                                group => $user->{group},
                                                lastlogin => $user->{lastlogin},
                                                host => $user->{host},
                                                browser => $user->{browser},
                                                ip => $user->{ip},
                                                pmail => $user->{privateemail},
                                                email => $user->{email},
                                        });
                return [$outputoff->restrictToCurrentUser];

                }
        }
}