###################################################################
#  GTChat 0.95 Alpha Plugin                                       #
#  Written for release 20020911                                   #
#  Author: Wladimir Palant etwas abgendert von Lord              #
# !!! Noch mal vielen Dank an Wladimir fr den grandiosen Chat!!! #
#                                                                 #
#  This plugin provides the chat commands /wackel                 #
###################################################################


package GTChat::Plugins::CommandHandlers::wackel;
use strict;

return bless({
	command_handlers => {
		'wackel' => \&commandHandler,
	},       
});

sub check_gagged
{
	my $main = shift;

	if (exists($main->{current_user}{gagtime}) && $main->{runtime}{now} < $main->{current_user}{gagtime})
	{
		return [$main->createErrorOutput('gagged',{seconds => $main->{current_user}{gagtime}-$main->{runtime}{now}})];
	}
	return undef;
}

sub commandHandler
{
	my($self,$main,$command,$text) = @_;

	if (my $error=check_gagged($main))
	{
		return $error;
	}
	
	my @parts = split(/\s+/,$text);
	my $tonick = shift @parts;
	$text = $main->toHTML(join(@parts));

	return [$main->createErrorOutput('msg_namenotgiven')] if (!defined($tonick));
	
	my $candidates = $main->getPossibleOnlineUsers($tonick);

	if ($#$candidates>0)    # Too many users online with nick starting like this
	{
		return [$main->createErrorOutput('ambiguousname',{nick => $tonick})];
	}
	elsif ($#$candidates==0)   # User is online
	{
		my $user = $candidates->[0];
		
		return [$main->createErrorOutput('msgtooneself')] if ($user->{name} eq $main->{current_user}{name});
                        
		my $output1 = $main->createOutput(
					{
						template => 'wackel_send',
						toname => $user->{name},
						tonick => $user->{nick},
						color => $main->{current_user}{color},
						text => ($command eq 'wackel'),
						url => ($command eq 'wackel'),
					});
		my $output2 = $main->createOutput(
					{
						template => 'wackel',
						name => $main->{current_user}{name},
						nick => $main->{current_user}{nick},
						color => $main->{current_user}{color},
						text => ($command eq 'wackel'),
						url => ($command eq 'wackel'),
					});
		return [$output1->restrictToCurrentUser, $output2->restrictToUser($user->{name})];
	}
	else                      # No user with this nickname online
	{
		$candidates = $main->getPossibleUsernames($tonick);

		if ($#$candidates>1)        # There are too many users with nick starting like this
		{
			return [$main->createErrorOutput('ambiguousname',{nick => $tonick})];
		}
		elsif ($#$candidates<0)     # No user with this nick found
		{
			return [$main->createErrorOutput('unknownname',{nick => $tonick})];
		}
		else                        # User was found but isn't online
		{
			my $toname = $candidates->[0];
			$tonick = $candidates->[1];

			my $line = $main->{now}.'|'.$main->toOutputString(
					{
						template => 'wackel',
						name => $main->{current_user}{name},
						nick => $main->{current_user}{nick},
						color => $main->{current_user}{color},
						text => ($command eq 'wackel'),
						url => ($command eq 'wackel'),
					});

			my $filename = unpack("H*",$toname);
			$main->open(local *FILE,'>>'.$main->translateName("memberdir::$filename.msg"));
			print FILE "$line\n";
			$main->close(*FILE);

			my $output = $main->createOutput(
					{
						template => 'offline_sent',
						toname => $toname,
						tonick => $tonick,
						color => $main->{current_user}{color},
						text => ($command eq 'wackel'),
						url => ($command eq 'wackel'),
					});
			return [$output->restrictToCurrentUser];
		}
	}
}
